PRO vs_simmidi, state
  COMMON vs_block
  IF NOT vs_debug THEN on_error, 2 ;return to caller on error (if not in debugging mode)
  ;;
  ;; Basic Data
  ;;
  NumBlines = state.NumBlines
  BaseLine = state.BaseLine
  BarID = vs_progbar(0, 0, Action='Reading Input File')
  ;;
  ;; Read Input File
  ;;
  Cube = readfits(state.InFile, CHead, silent)
  IF total(Cube) EQ -1 THEN BEGIN
      vs_info, 'File '+state.InFile+' not found!', 4
      return
  ENDIF 
  NumAxis = sxpar(CHead, 'NAXIS')
  Isize = sxpar(CHead, 'NAXIS1')
  Ysize = sxpar(CHead, 'NAXIS2')
  IF Isize NE Ysize THEN BEGIN
      vs_info, 'Can only work on square images', 2
      return
  ENDIF 
  JVec = findgen(Isize)
  IF NumAxis GT 2 THEN $
    NumFrames=sxpar(CHead, 'NAXIS3') $
  ELSE $
    NumFrames=1
  PixSize = sxpar(CHead, 'PIXSIZE')
  Lambda = fltarr(NumFrames)    ; Now extract wavelengths & comments
  Comment = strarr(NumFrames)
  FOR i=0, NumFrames-1 DO BEGIN
      IF vs_debug THEN print, strcompress('LAMBDA'+string(format='(i3)', i+1), /rem)
      IF vs_debug THEN print, strcompress('TAG'+string(format='(i3)', i+1), /rem)
      
      lambda[i] = sxpar(CHead, strcompress('LAMBDA'+string(format='(i3)', i+1), /rem))
      TagTmp = sxpar(CHead, strcompress('TAG'+string(format='(i3)', i+1), /rem))
      IF !ERR NE -1 THEN Comment[i] =  TagTmp
  ENDFOR 
  ;;
  ;; Compute visibilities
  ;;
  Dummy = VS_Progbar(BarID, 25, Action='Computing Visibilities...')
  Visib = fltarr(Isize, Isize, NumFrames)
  FOR i=0, NumFrames-1 DO BEGIN
      IF state.Rot NE 0 THEN Cube[*, *, i] = rot(Cube[*, *, i], state.Rot)
      FT = fft(Cube[*, *, i], -1) ;fourier transform 
      Visi = shift(abs(FT)/abs(FT[0, 0]), Isize/2., Isize/2.) ;normalize and shift visibility
      Visib[*, *, i] = Visi 
  ENDFOR 
  ;;
  ;; Compute baseline tracks
  ;;
  Dummy = VS_Progbar(BarID, 50, Action='Computing Baseline Tracks...')
  BaseTracks = fltarr(NumBlines, Isize, 3, NumFrames)
  IndTracks =  fltarr(NumBlines, Isize, 2, NumFrames)
  FOR i=0, NumFrames-1 DO BEGIN 
      BT = vs_baseproj(state.ObjDec, $
                       state.StartHr, $
                       state.EndHr, $
                       *state.BaseLine, $
                       state.NumBlines, $
                       lambda[i], $
                       Isize)
      BaseTracks[*, *, *, i] = BT
      IndTracks[*, *, 0:1, i] = BaseTracks[*, *, 0:1, i]*Isize*PixSize+Isize/2
  ENDFOR 
  ;;
  ;; Now make hourangle-freequency-visibility array
  ;;
  Dummy = VS_Progbar(BarID, 75, Action='Computing Result Array...')
  HFV = fltarr(NumBlines, Isize, 3, NumFrames)
  IF state.FilterW EQ 0 THEN BEGIN
      ;; simple case with infinite spectral resolution
      FOR i=0, NumFrames-1 DO BEGIN
          FOR j=0, NumBlines-1 DO BEGIN 
              HFV[j, *, 0, i] = BaseTracks[j, *, 2, i]
              HFV[j, *, 1, i] = sqrt(BaseTracks[j, *, 0, i]^2+BaseTracks[j, *, 1, i]^2)
              HFV[j, *, 2, i] = interpolate(Visib[*, *, i], $
                                            IndTracks[j, JVec, 0, i], $
                                            IndTracks[j, JVec, 1, i])
          ENDFOR 
      ENDFOR 
  ENDIF ELSE BEGIN
      ;; Complicated case with averaging over 10 wavelengths
      NumSub = 10
      SubTracks = fltarr(NumBlines, Isize, 3, NumFrames, NumSub)
      SubIndTracks =  fltarr(NumBlines, Isize, 2, NumFrames, NumSub)
      FOR i=0, NumFrames-1 DO BEGIN ;Compute baseline tracks for sub-wavelengths
          SubLambdas = findgen(10)*Lambda[i]*state.FilterW+Lambda[i]-Lambda[i]*state.FilterW/2
          FOR j=0, NumSub-1 DO BEGIN 
              BT = vs_baseproj(state.ObjDec, $
                               state.StartHr, $
                               state.EndHr, $
                               *state.BaseLine, $
                               state.NumBlines, $
                               SubLambdas[j], $
                               Isize)
              SubTracks[*, *, *, i, j] = BT
              SubIndTracks[*, *, 0:1, i, j] = SubTracks[*, *, 0:1, i, j]*Isize*PixSize+Isize/2
          ENDFOR 
      ENDFOR 
      FOR i=0, NumFrames-1 DO BEGIN
          FOR j=0, NumBlines-1 DO BEGIN 
              HFV[j, *, 0, i] = BaseTracks[j, *, 2, i]
              HFV[j, *, 1, i] = sqrt(BaseTracks[j, *, 0, i]^2+BaseTracks[j, *, 1, i]^2)
              HelpVisi = fltarr(Isize, NumSub)
              FOR k=0, NumSub-1 DO BEGIN ;read visibilities along all sub-tracks
                  HelpVisi[*, k] = interpolate(Visib[*, *, i], $
                                               SubIndTracks[j, JVec, 0, i, k], $
                                               SubIndTracks[j, JVec, 1, i, k])
              ENDFOR 
              HFV[j, *, 2, i] = total(HelpVisi, 2)/NumSub ;average all sub-visibilities
          ENDFOR 
      ENDFOR 
  ENDELSE 
  ;;
  ;; Store away result
  ;;
  state.Result = ptr_new({Cube:Cube, $
                          Visib:Visib, $
                          BaseTracks:BaseTracks, $
                          IndTracks:IndTracks, $
                          HFV:HFV, $
                          PixSize:PixSize, $
                          Lambda:Lambda, $
                          Comment:Comment $
                         })
  ;;
  ;; Finished
  ;;
  Dummy = VS_Progbar(BarID, -1)
END 
